@@ -2,7 +2,8 @@ from __future__ import annotations
 
 import functools
 import itertools
-from typing import Callable, Generator, Generic, Iterable, Self, Sequence, TypeVar
+from collections.abc import Callable, Generator, Iterable, Iterator, Sequence, Sized
+from typing import Generic, Self, TypeVar
 
 __all__ = [
     "head",
@@ -26,6 +27,7 @@ T = TypeVar("T")
 K = TypeVar("K")
 U = TypeVar("U")
 V = TypeVar("V")
+Tco = TypeVar("Tco", covariant=True)
 
 
 # See Haskell's list operations head, tail, init, and last
@@ -185,8 +187,8 @@ def merge_chunks(
     return merged_chunks
 
 
-class Seq(Generic[T]):
-    def __init__(self, data: Iterable[T] | Self):
+class Seq(Generic[Tco], Sequence[Tco], Sized):
+    def __init__(self, data: Iterable[Tco] | Self):
         if isinstance(data, Seq):
             self.data = data.data
         elif isinstance(data, Iterable):
@@ -195,12 +197,12 @@ class Seq(Generic[T]):
             raise ValueError("unsupported data type")
 
     @property
-    def count(self) -> int:
+    def size(self) -> int:
         return len(self.data)
 
     @property
     def empty(self) -> bool:
-        return self.count == 0
+        return self.size == 0
 
     def __add__(self, other: Self) -> Self:
         return self.concat(other)
@@ -212,6 +214,24 @@ class Seq(Generic[T]):
             return Seq([self.data[item]])
         raise ValueError("unsupported index type")
 
+    def __len__(self) -> int:
+        return len(self.data)
+
+    def __contains__(self, item) -> bool:
+        return item in self.data
+
+    def __iter__(self) -> Iterator[Tco]:
+        return iter(self.data)
+
+    def __reversed__(self) -> Iterator[Tco]:
+        return reversed(self.data)
+
+    def count(self, elem: Tco):
+        return self.count_if(lambda x: x == elem)
+
+    def count_if(self, func: Callable[[Tco], bool]):
+        return sum(1 for item in self.data if func(item))
+
     def concat(self, other: Self) -> Self:
         return Seq(self.data + other.data)
 
@@ -229,7 +249,7 @@ class Seq(Generic[T]):
     def distinct(self) -> Self:
         return Seq(list(set(self.data)))
 
-    def scan_left(self, zero: U | None, func: Callable[[U, T], U]) -> "Seq[U]":
+    def scan_left(self, zero: U | None, func: Callable[[U, Tco], U]) -> "Seq[U]":
         def scan():
             accum = zero
             for elem in self.data:
@@ -238,7 +258,7 @@ class Seq(Generic[T]):
 
         return Seq(scan())
 
-    def scan_right(self, zero: U | None, func: Callable[[U, T], U]) -> "Seq[U]":
+    def scan_right(self, zero: U | None, func: Callable[[U, Tco], U]) -> "Seq[U]":
         def scan():
             accum = zero
             for elem in reversed(self.data):
@@ -249,10 +269,10 @@ class Seq(Generic[T]):
 
     scan = scan_left
 
-    def map(self, func: Callable[[T], U]) -> "Seq[U]":
+    def map(self, func: Callable[[Tco], U]) -> "Seq[U]":
         return self.scan(None, lambda x, y: func(y))
 
-    def fold_left(self, zero: U | None, func: Callable[[U, T], U]) -> "Seq[U]":
+    def fold_left(self, zero: U | None, func: Callable[[U, Tco], U]) -> "Seq[U]":
         if self.empty:
             return Seq([]) if zero is None else Seq([zero])
         data = self.data if zero is None else [zero] + self.data
@@ -261,7 +281,7 @@ class Seq(Generic[T]):
             accum = func(accum, elem)
         return Seq([accum])
 
-    def fold_right(self, zero: U | None, func: Callable[[U, T], U]) -> "Seq[U]":
+    def fold_right(self, zero: U | None, func: Callable[[U, Tco], U]) -> "Seq[U]":
         if self.empty:
             return Seq([]) if zero is None else Seq([zero])
         data = self.data if zero is None else self.data + [zero]
@@ -272,18 +292,18 @@ class Seq(Generic[T]):
 
     fold = fold_left
 
-    def reduce(self, func: Callable[[T, T], T]) -> Self:
+    def reduce(self, func: Callable[[Tco, Tco], Tco]) -> Self:
         return self.fold(None, lambda x, y: func(x, y))
 
-    def max(self, func: Callable[[T, T], bool] = None) -> Self:
+    def max(self, func: Callable[[Tco, Tco], bool] = None) -> Self:
         func = func or (lambda x, y: x > y)
         return self.reduce(lambda x, y: x if func(x, y) else y)
 
-    def min(self, func: Callable[[T, T], bool] = None) -> Self:
+    def min(self, func: Callable[[Tco, Tco], bool] = None) -> Self:
         func = func or (lambda x, y: x < y)
         return self.reduce(lambda x, y: x if func(x, y) else y)
 
-    def group(self, func: Callable[[T], K]) -> "Seq[tuple[K, list[T]]]":
+    def group(self, func: Callable[[Tco], K]) -> "Seq[tuple[K, list[Tco]]]":
         return Seq(grouped(self.data, key_func=func, keys_ordered=True))
 
     def keys(self):
@@ -295,10 +315,10 @@ class Seq(Generic[T]):
     def swap(self):
         return Seq((value, key) for key, value in self.data)
 
-    def map_keys(self, func: Callable[[T], U]) -> Self:
+    def map_keys(self, func: Callable[[Tco], U]) -> Self:
         return Seq((func(key), value) for key, value in self.data)
 
-    def map_values(self, func: Callable[[T], U]) -> Self:
+    def map_values(self, func: Callable[[Tco], U]) -> Self:
         return Seq((key, func(value)) for key, value in self.data)
 
     def flatten(self):
@@ -307,19 +327,19 @@ class Seq(Generic[T]):
             data.extend(d)
         return Seq(data)
 
-    def flat_map(self, func: Callable[[T], U]) -> "Seq[U]":
+    def flat_map(self, func: Callable[[Tco], U]) -> "Seq[U]":
         return self.flatten().map(func)
 
-    def group_map(self, group_func: Callable[[T], K], map_func: Callable[[T], U]) -> "Seq[tuple[K, list[U]]]":
+    def group_map(self, group_func: Callable[[Tco], K], map_func: Callable[[Tco], U]) -> "Seq[tuple[K, list[U]]]":
         return self.group(group_func).map_values(lambda x: list(map(map_func, x)))
 
-    def filter(self, func: Callable[[T], bool]) -> Self:
+    def filter(self, func: Callable[[Tco], bool]) -> Self:
         return Seq(filter(func, self.data))
 
-    def filter_not(self, func: Callable[[T], bool]) -> Self:
+    def filter_not(self, func: Callable[[Tco], bool]) -> Self:
         return self.filter(lambda x: not func(x))
 
-    def sort(self, func: Callable[[T], K]) -> Self:
+    def sort(self, func: Callable[[Tco], K]) -> Self:
         return Seq(sorted(self.data, key=func))
 
     def head(self) -> Self:
@@ -334,15 +354,15 @@ class Seq(Generic[T]):
     def tail(self) -> Self:
         return Seq(tail(self.data))
 
-    def foreach(self, func: Callable[[T], None]) -> Self:
+    def foreach(self, func: Callable[[Tco], None]) -> Self:
         for elem in self.data:
             func(elem)
         return self
 
-    def exists(self, func: Callable[[T], bool]) -> "Seq[bool]":
+    def exists(self, func: Callable[[Tco], bool]) -> "Seq[bool]":
         return Seq([any(map(func, self.data))])
 
-    def forall(self, func: Callable[[T], bool]) -> "Seq[bool]":
+    def forall(self, func: Callable[[Tco], bool]) -> "Seq[bool]":
         return Seq([all(map(func, self.data))])
 
     def union(self, other: Self) -> Self:
@@ -351,5 +371,11 @@ class Seq(Generic[T]):
     def intersect(self, other: Self) -> Self:
         return Seq(list(set(self.data).intersection(set(other.data))))
 
+    def zip(self, other: Seq[U]) -> Seq[tuple[Tco, U]]:
+        return Seq(zip(self.data, other.data))
+
+    def zip_fill(self, other: Seq[U], fill: Tco | U | None = None) -> Seq[tuple[Tco, U]]:
+        return Seq(itertools.zip_longest(self.data, other.data, fillvalue=fill))
+
 
 seq = Seq
